<?php

/**
 * API pour l'interrogation d'Opens Datas
 *
 * Cette API interroge les Opens Datas pertinents
 * pour y trouver les contours des Subdivisions.
 * Elle peuple la table spip_gis de la forme géométrique,
 * établit le lien avec l'objet subdivision dans spip_gis_liens
 * et garde une trace de cette opération.
 *
 * API à utiliser :
 *     ```
 *     include_spip('inc/apigeo');
 *     ```
 * @plugin     Subdivisions Géographiques
 * @copyright  2019-2020
 * @author     Vincent CALLIES
 * @licence    GNU/GPL
 * @package    SPIP\Subdivisions\inc
 */

// Sécurité
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Correspondance du code iso à une subdivision et une API
 *
 * @api
 * @param string $code_iso
 *     Code ISO
 * @return bool|array
 *     - false si la subdivision ou le code ISO n'a pas de forme géométrique disponible en Open Data.
 *       - [id_subdivision_api] clé primaire de la table spip_subdivisions_apis, si on y trouve l'API Open Data pertinente
 *		 - [id_subdivision] clé primaire de la table spip_subdivisions.
 */

function apigeo_correspondances_iso ($code_iso){
	// vérification basique pour s'assurer que l'on ait à faire à un code iso
	if (!preg_match('/^([A-Z]{2})-(\d{2}|\d{1}[A-Z]{1}|[A-Z]{2,3})$/', $code_iso)) {
		return false;
	}
	// il n'y a pas de FULL JOIN avec MySQL	
	return sql_fetsel(
		"a.id_subdivision_api,s.id_subdivision",
		"spip_subdivisions AS s 
			LEFT JOIN spip_subdivisions_apis AS a 
			ON (s.iso_moi=a.iso_moi)",
		array("s.iso_moi=" . sql_quote($code_iso))
		);

}

/**
 * Trouve l'API en Open Data pour une subdivision ou un code ISO
 *
 * @api
 * @param int|string $id_subdivision|$code_iso
 *     Identifiant unique de la subdivision ou Code ISO
 * @return bool|int
 *     - false si la subdivision ou le code ISO n'a pas de forme géométrique disponible en Open Data.
 *     - clé primaire de la table spip_subdivisions_apis, si on y trouve l'API Open Data pertinente.
 */

function apigeo_trouver_api ($argument){
	if (intval($argument)) {
		return sql_getfetsel(
			"a.id_subdivision_api",
			"spip_subdivisions AS s 
				LEFT JOIN spip_subdivisions_apis AS a 
				ON s.iso_moi=a.iso_moi",
			"s.id_subdivision=" . intval($argument)
		);
	} elseif (preg_match('/^([A-Z]{2})-(\d{2}|\d{1}[A-Z]{1}|[A-Z]{2,3})$/', $argument)) {
		//  ISO 3166-2 code du pays à deux lettres + - + deux ou trois caractères lettre majuscule ou nombre.
		return sql_getfetsel("id_subdivision_api", "spip_subdivisions_apis", "iso_moi=" . sql_quote($argument));
	}

	return false;
}


/**
 * Teste si une subdivision dispose d'une forme géométrique déjà donnée par une API
 *
 * @api
 * @param int $id_subdivision
 *     Identifiant unique de la subdivision à tester
 * @param string|null $relation
 *     Qualification de la forme associée à la subdivision
 * @return bool|string
 *     - false la subdivision n'a pas de forme peuplant la la base de données.
 *     - date_peuplement, si on y trouve la forme.
 */

function apigeo_trouver_forme ($id_subdivision, $type_relation = ""){
	// La subdivision a-t-elle déjà une forme géométrique provenant d'une API ?
	$conditions = array (
		"id_subdivision=" . intval($id_subdivision),
		"statut=" . sql_quote('publie')
	);

	if ($type_relation) {
		$conditions = array_merge($conditions, array("type_relation=" . sql_quote($type_relation)));
	}
			
	return sql_getfetsel("date_peuplement", "spip_subdivisions_peuplements", $conditions);
}

/**
 * Contruit la requête permettant d'obtenir de l'API une forme géométrique
 *
 * @api
 * @param int $id_subdivision_api
 *     Identifiant unique de l'API a mettre en oeuvre
 * @return bool|string
 *     - false la subdivision n'a pas de forme peuplant la la base de données.
 *     - date_peuplement, si on y trouve la forme.
 */

function apigeo_construire_requete ($id_subdivision_api) {
	// appel de l'annuaire des apis
	if ($row = sql_fetsel("*", "spip_subdivisions_apis", "id_subdivision_api=" . intval($id_subdivision_api))){
		// construction de la requete
		$url = parametre_url($row['api'], 'dataset', $row['dataset'], '&');
		$url = parametre_url($url, 'q', '', '&');
		$url = parametre_url($url, 'facet', $row['facet'], '&');
		$url = parametre_url($url, 'refine.'.$row['facet'], $row['refine'], '&');
	} else {
		$url = false;
	}
	return $url;
}

/**
 * Trouver le noeud 
 *
 * @api
 * @param int $id_subdivision_api
 *     Identifiant unique de l'API indiquant notamment le noeud
 * @param json $json
 *     Données json dans lesquelle il faudra identifier le noeud
 * @return bool|string
 *     - false car il n'y a pas de données.
 *     - array des données json simplifiées.
 */

function trouver_noeud($objet,$noeud) {
   	foreach ($objet as $key => $value)
       {
   		if ($key == $noeud) {
   			return $value;
      	}
	}
}

/**
 * Extrait des données json le noeud d'informations pertinentes
 *
 * @api
 * @param int $id_subdivision_api
 *     Identifiant unique de l'API indiquant notamment le noeud
 * @param json $json
 *     Données json dans lesquelle il faudra identifier le noeud
 * @return bool|string
 *     - false car il n'y a pas de données.
 *     - array des données json simplifiées.
 */

function apigeo_extraire ($id_subdivision_api,$json) {

	// récupérer le noeud d'informations pertinentes		
	$noeud = sql_getfetsel("noeud", "spip_subdivisions_apis", array (
			"id_subdivision_api=" . intval($id_subdivision_api)));
	$corde = explode('/', $noeud);
	$profondeur = count($corde);

	// fouiller l'objet pour retrouver le noeud
	$objet = json_decode($json, true); 

	
	$i = 0; 
	while ($i <= $profondeur-1) {
		$objet = trouver_noeud($objet,$corde[$i]);
    	$i++;
	}

	// renvoyer l'information pertinente en json
	return json_encode($objet);

}

/**
 * Enregistrer des données json dans la base de données
 *
 * @api
 * @param json $donnees
 *     Données json a enregistrée (objet géométrique)
 * @param json $id_subdivision
 *     Pourquoi ?
 * @return bool|string
 *     - false car il n'y a pas de données.
 *     - json des données json simplifiées.
 */

function apigeo_peupler($donnees, $id_subdivision) {
	$id_gis = false;
	$f = find_in_path(_DIR_LIB_GEOPHP.'geoPHP.inc', '', true);
	if ($f) {
		// qualifier les informations

		// charger les données principales constituant l'objet géométrique
		$geometry = geoPHP::load($donnees, 'json');
		$i['geo'] = $geometry->out('json');
		// donner un titre et un descriptif à partir de l'API
		$info = sql_fetsel(
			"a.id_subdivision_api, CONCAT(a.dataset,' - ', a.titre) AS descriptif, s.titre",
			"spip_subdivisions AS s 
				LEFT JOIN spip_subdivisions_apis AS a 
				ON s.iso_moi=a.iso_moi",
			"s.id_subdivision=" . intval($id_subdivision)
		);
		$i['titre'] = $info['titre'];
		$i['descriptif'] = $info['descriptif'];
		$id_subdivision_api = $info['id_subdivision_api'];
		unset($info); 
		// renseigner les coordonnées de l'objet à partir de son centroid
		$centroid = $geometry->getCentroid();
		$i['lat'] = $centroid->getY();
		$i['lon'] = $centroid->getX();
		$i['type'] = $geometry->geometryType();
		
		// enregistrer ces informations dans spip_gis

		include_spip('gisgeom_fonctions');
		$wkt = json_to_wkt($i['geo']);
		// convertir le WKT en binaire avant l'insertion
		$i['geo'] = sql_getfetsel("GeomFromText('$wkt')");
		if ($id_gis = sql_insertq('spip_gis', $i)) {
 			spip_log("Fonction importer_spip_gis."
 				. "(" . $i['titre'] . ")"
 				. " La forme id_gis n°$id_gis est liée avec id_subdivision n°$id_subdivision.",
				'subdivisions.' . _LOG_INFO
			);

		// établir la liaison entre la forme enregistrée dans spip_gis et la subdivision enregistrée dans spip_subdivision

			sql_insertq('spip_gis_liens', array(
				'id_gis' => $id_gis,
				'objet' => 'subdivision',
				'id_objet' => $id_subdivision,
				)
			);

		// garder une trace de ce peuplement et de ses conditions pour permettre une mise à jour.

			$champs = array (
				'id_subdivision_api' => $id_subdivision_api,
				'date_peuplement' => date('Y-m-d H:i:s'),
				'id_subdivision' => $id_subdivision,
				'id_gis' => $id_gis,
				'type_relation' => "definition",
				'date' => date('Y-m-d H:i:s'),
				'statut' => "publie"
			);
			$id_peuplement = sql_insertq('spip_subdivisions_peuplements', $champs);
 			spip_log("Fonction importer_spip_gis."
 				. "(" . $i['titre'] . ")"
 				. " La forme id_gis n°$id_gis est liée avec id_subdivision n°$id_subdivision est tracée id.",
				'subdivisions.' . _LOG_INFO
			);

		}
	}
	return $id_gis;
}

